// -------------------------------------------------------------------------
//    
//
//  
// -------------------------------------------------------------------------
#include "StdAfx.h"
#include "BrkWinArea.h"

// 
CBrkWinArea::CBrkWinArea(WORD AreaID, POINT AreaPos, SIZE AreaSize, CBrokenWinCore* pBrokenWin)
{
	m_AreaID = AreaID;
	m_Pos = AreaPos;
	m_Size = AreaSize;
	m_pBrokenWin = pBrokenWin;

	m_hCompDC = NULL;
	m_hBitMap = NULL;
	m_hOldBitMap = NULL;
}

// 
CBrkWinArea::~CBrkWinArea(void)
{
	if (m_hCompDC != NULL)
	{
		if (m_hBitMap != NULL)
		{
			SelectObject(m_hCompDC,m_hOldBitMap);
			DeleteObject(m_hBitMap);
		}
		DeleteDC(m_hCompDC);
	}
}

//    
BOOL CBrkWinArea::PreparationDC(HDC hDC)
{
	if (m_Size.cx == 0 || m_Size.cy == 0)
		return FALSE;

	//    
	if (!m_hCompDC)
		m_hCompDC = CreateCompatibleDC(hDC);

	//    
	if (!m_hBitMap)
	{
		m_hBitMap = CreateCompatibleBitmap(hDC,m_Size.cx,m_Size.cy);
		m_hOldBitMap = (HBITMAP)SelectObject(m_hCompDC,m_hBitMap);
	}

	return TRUE;
}

//      
void CBrkWinArea::GetAreaRect(RECT* pAreaRect)
{
	SetRect(pAreaRect, m_Pos.x, m_Pos.y, m_Pos.x + m_Size.cx, m_Pos.y + m_Size.cy);
}

//      (    )
BOOL CBrkWinArea::PtInArea(POINT Pt)
{
	RECT AreaRect;
	GetAreaRect(&AreaRect);

	return PtInRect(&AreaRect,Pt);
}

//   
void CBrkWinArea::ProccesIntersect()
{
	_ASSERT(m_pBrokenWin != NULL);

	//   
	RECT SelfRect;
	SetRect(&SelfRect, m_Pos.x, m_Pos.y, m_Pos.x + m_Size.cx, m_Pos.y + m_Size.cy);
	//    
	UINT BelowAreasCount = m_pBrokenWin->m_BrkWinAreas.size();

	//  
	for (UINT i = 0; i < BelowAreasCount; i++)
	{
		CBrkWinArea* pBelowArea = m_pBrokenWin->m_BrkWinAreas[i];
		_ASSERT(pBelowArea != NULL);
		//    
		RECT BelowRect;
		pBelowArea->GetAreaRect(&BelowRect);

		//    
		RECT InterRect;
		if (IntersectRect(&InterRect,&BelowRect,&SelfRect))
		{//  
			INTERSECT_DATA IntersectData;
			IntersectData.BelowAreaIdx = i;
			CopyRect(&IntersectData.BelowInterRect,&InterRect);
			OffsetRect(&IntersectData.BelowInterRect,-BelowRect.left,-BelowRect.top);
			CopyRect(&IntersectData.SelfInterRect,&InterRect);
			OffsetRect(&IntersectData.SelfInterRect,-m_Pos.x,-m_Pos.y);
			m_InersectData.push_back(IntersectData);
		}
	}
}

//    
void CBrkWinArea::CopyIntersContent(RECT SourceRect, HDC hDestDC, RECT DestRect)
{
	BitBlt(hDestDC,DestRect.left,DestRect.top,
			 DestRect.right - DestRect.left,DestRect.bottom - DestRect.top,
			 m_hCompDC,SourceRect.left,SourceRect.top,SRCCOPY);
}

//  
void CBrkWinArea::Draw(HDC hDC, BOOL bUpdateAreaImage)
{
	_ASSERT(m_pBrokenWin != NULL);

	if (!m_hCompDC || bUpdateAreaImage)
	{//          
		//    
		if (!PreparationDC(hDC))
			return;

		//    
		m_pBrokenWin->DrawBGforArea(m_hCompDC, m_Pos, m_Size);
		//      
		for (UINT i = 0; i < m_InersectData.size(); i++)
		{
			m_pBrokenWin->m_BrkWinAreas[m_InersectData[i].BelowAreaIdx]->CopyIntersContent(m_InersectData[i].BelowInterRect,
				m_hCompDC,m_InersectData[i].SelfInterRect);
		}

		//   
		DrawEvent(m_hCompDC);
	}

	//      
	BitBlt(hDC,m_Pos.x,m_Pos.y,m_Size.cx,m_Size.cy,m_hCompDC,0,0,SRCCOPY);
}

//   
WORD CBrkWinArea::GetAreaID()
{
	return m_AreaID;
}
