// -------------------------------------------------------------------
//      
//
//  
// -------------------------------------------------------------------
#include "StdAfx.h"
#include "BrokenWinCore.h"
#include "BrkWinResData.h"

// ---------------------------------------------------------
//  
// ---------------------------------------------------------
//    
#define BRK_WIN_CLASS_NAME			_T("BrokenWindow")
//       
#define BRK_WIN_CLASS_PROP			_T("BRK_WIN_CLASS_PROP")
// ---------------------------------------------------------

// 
CBrokenWinCore::CBrokenWinCore(void)
{
	m_hWnd					= NULL;
	m_hBGDC					= NULL;
	m_hBGBitMap				= NULL;
	m_hBGOldBitMap			= NULL;
	m_iMovePrevAreaIdx	= -1;
	m_CaptureAreaIdx		= -1;
}

// 
CBrokenWinCore::~CBrokenWinCore(void)
{
	//   
	ClearAreas();
	//    
	DestroyBGDC();

	//  
	if (m_hWnd != NULL)
		DestroyWindow(m_hWnd);
}

//    
ATOM CBrokenWinCore::BrkWinRegisterClass(HINSTANCE hInstance)
{
	WNDCLASSEX wcex;

	wcex.cbSize = sizeof(WNDCLASSEX);

	wcex.style				= 0;
	wcex.lpfnWndProc		= BrkWinWndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance			= hInstance;
	wcex.hIcon				= NULL;
	wcex.hCursor			= ::LoadCursor(NULL,IDC_ARROW);
	wcex.hbrBackground	= NULL;
	wcex.lpszMenuName		= NULL;
	wcex.lpszClassName	= BRK_WIN_CLASS_NAME;
	wcex.hIconSm			= NULL;

	return ::RegisterClassEx(&wcex);
}

//    
LRESULT CALLBACK CBrokenWinCore::BrkWinWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	//  
	case WM_CREATE:
		{
			LPCREATESTRUCT lpcs = (LPCREATESTRUCT)lParam;
			CBrokenWinCore* pBrkWinCore = (CBrokenWinCore*)lpcs->lpCreateParams;
			_ASSERT(pBrkWinCore != NULL);
			return pBrkWinCore->Create(hWnd,lpcs);
		}
	//   
	case WM_ERASEBKGND:
		return TRUE;
	//    
	case WM_LBUTTONDOWN:
		{
			CBrokenWinCore* pBrkWinCore = (CBrokenWinCore*)::GetProp(hWnd,BRK_WIN_CLASS_PROP);
			_ASSERT(pBrkWinCore != NULL);
			UINT nFlags = wParam;
			POINT PosPt;
			PosPt.x = GET_X_LPARAM(lParam);
			PosPt.y = GET_Y_LPARAM(lParam);
			pBrkWinCore->LButtonDown(FALSE,nFlags,PosPt);
		}
		break;
	//     
	case WM_MOUSELEAVE:
		{
			CBrokenWinCore* pBrkWinCore = (CBrokenWinCore*)::GetProp(hWnd, BRK_WIN_CLASS_PROP);
			_ASSERT(pBrkWinCore != NULL);
			pBrkWinCore->MouseLeave();
		}
		break;
	//  
	case WM_MOUSEMOVE:
		{
			CBrokenWinCore* pBrkWinCore = (CBrokenWinCore*)::GetProp(hWnd,BRK_WIN_CLASS_PROP);
			_ASSERT(pBrkWinCore != NULL);
			UINT nFlags = wParam;
			POINT PosPt;
			PosPt.x = GET_X_LPARAM(lParam);
			PosPt.y = GET_Y_LPARAM(lParam);
			pBrkWinCore->MouseMove(nFlags,PosPt);
		}
		break;
	//    
	case WM_LBUTTONUP:
		{
			CBrokenWinCore* pBrkWinCore = (CBrokenWinCore*)::GetProp(hWnd,BRK_WIN_CLASS_PROP);
			_ASSERT(pBrkWinCore != NULL);
			UINT nFlags = wParam;
			POINT PosPt;
			PosPt.x = GET_X_LPARAM(lParam);
			PosPt.y = GET_Y_LPARAM(lParam);
			pBrkWinCore->LButtonUp(nFlags,PosPt);
		}
		break;
		//     
	case WM_RBUTTONDOWN:
		{
			CBrokenWinCore* pBrkWinCore = (CBrokenWinCore*)::GetProp(hWnd, BRK_WIN_CLASS_PROP);
			_ASSERT(pBrkWinCore != NULL);
			UINT nFlags = wParam;
			POINT PosPt;
			PosPt.x = GET_X_LPARAM(lParam);
			PosPt.y = GET_Y_LPARAM(lParam);
			pBrkWinCore->RButtonDown(FALSE, nFlags, PosPt);
		}
		break;
	//    
	case WM_CAPTURECHANGED:
		{
			//  
			::InvalidateRect(hWnd,NULL,TRUE);
			::UpdateWindow(hWnd);
		}
		return 0;
	//  
	case WM_PAINT:
		{
			PAINTSTRUCT ps;
			HDC hDC = BeginPaint(hWnd, &ps);
			//   
			CBrokenWinCore* pBrkWinCore = (CBrokenWinCore*)::GetProp(hWnd,BRK_WIN_CLASS_PROP);
			_ASSERT(pBrkWinCore != NULL);
			pBrkWinCore->Draw(hDC);
			//  
			EndPaint(hWnd, &ps);
		}
		break;
	//  
	case WM_SETCURSOR:
		{
			CBrokenWinCore* pBrkWinCore = (CBrokenWinCore*)::GetProp(hWnd,BRK_WIN_CLASS_PROP);
			_ASSERT(pBrkWinCore != NULL);
			if (pBrkWinCore->SetCursorEvent())
				return TRUE;
		}
		return ::DefWindowProc(hWnd, WM_SETCURSOR, wParam, lParam);
	//   
	case WM_CONTEXTMENU:
		{
			CBrokenWinCore* pBrkWinCore = (CBrokenWinCore*)::GetProp(hWnd,BRK_WIN_CLASS_PROP);
			_ASSERT(pBrkWinCore != NULL);
			pBrkWinCore->ContextMenu(GET_X_LPARAM(lParam),GET_Y_LPARAM(lParam));
		}
		break;
	// 
	case WM_COMMAND:
		{
			CBrokenWinCore* pBrkWinCore = (CBrokenWinCore*)::GetProp(hWnd,BRK_WIN_CLASS_PROP);
			_ASSERT(pBrkWinCore != NULL);
			pBrkWinCore->CommandEvent((int)(LOWORD(wParam)), (HWND)(lParam), (UINT)HIWORD(wParam));
		}
		break;
	//   
	case WM_WINDOWPOSCHANGED:
		{
			CBrokenWinCore* pBrkWinCore = (CBrokenWinCore*)::GetProp(hWnd,BRK_WIN_CLASS_PROP);
			_ASSERT(pBrkWinCore != NULL);
			pBrkWinCore->WindowPosChangedEvent((LPWINDOWPOS)lParam);
		}
		break;
	//  
	case WM_TIMER:
		{
			CBrokenWinCore* pBrkWinCore = (CBrokenWinCore*)::GetProp(hWnd,BRK_WIN_CLASS_PROP);
			_ASSERT(pBrkWinCore != NULL);
			pBrkWinCore->TimerEvent((UINT)wParam);
		}
		break;
	//    (WM_APP+1)
	case WM_APP+1:
		{
			CBrokenWinCore* pBrkWinCore = (CBrokenWinCore*)::GetProp(hWnd,BRK_WIN_CLASS_PROP);
			_ASSERT(pBrkWinCore != NULL);
			pBrkWinCore->AppMessage1Event(wParam,lParam);
		}
		break;
	//    (WM_APP+2)
	case WM_APP+2:
		{
			CBrokenWinCore* pBrkWinCore = (CBrokenWinCore*)::GetProp(hWnd,BRK_WIN_CLASS_PROP);
			_ASSERT(pBrkWinCore != NULL);
			pBrkWinCore->AppMessage2Event(wParam,lParam);
		}
		break;
	//    (WM_APP+3)
	case WM_APP+3:
		{
			CBrokenWinCore* pBrkWinCore = (CBrokenWinCore*)::GetProp(hWnd,BRK_WIN_CLASS_PROP);
			_ASSERT(pBrkWinCore != NULL);
			pBrkWinCore->AppMessage3Event(wParam,lParam);
		}
		break;
	//    (WM_APP+4)
	case WM_APP+4:
		{
			CBrokenWinCore* pBrkWinCore = (CBrokenWinCore*)::GetProp(hWnd,BRK_WIN_CLASS_PROP);
			_ASSERT(pBrkWinCore != NULL);
			pBrkWinCore->AppMessage4Event(wParam,lParam);
		}
	//  
	case WM_CLOSE:
		{
			CBrokenWinCore* pBrkWinCore = (CBrokenWinCore*)::GetProp(hWnd,BRK_WIN_CLASS_PROP);
			_ASSERT(pBrkWinCore != NULL);
			pBrkWinCore->CloseEvent();
		}
		break;
	//  
	case WM_DESTROY:
		{
			CBrokenWinCore* pBrkWinCore = (CBrokenWinCore*)::GetProp(hWnd,BRK_WIN_CLASS_PROP);
			_ASSERT(pBrkWinCore != NULL);
			pBrkWinCore->Destroy();
		}
		break;
	//   Window
	case WM_ENDSESSION:
		{
			CBrokenWinCore* pBrkWinCore = (CBrokenWinCore*)::GetProp(hWnd,BRK_WIN_CLASS_PROP);
			_ASSERT(pBrkWinCore != NULL);
			pBrkWinCore->EndSessionEvent();
		}
		break;
	// /   
	case WM_POWERBROADCAST:
		{
			CBrokenWinCore* pBrkWinCore = (CBrokenWinCore*)::GetProp(hWnd,BRK_WIN_CLASS_PROP);
			_ASSERT(pBrkWinCore != NULL);
			pBrkWinCore->PowerBroadCastEvent(wParam);
			return TRUE;
		}
		break;
	//    
	case WM_DEVICECHANGE:
		{
			CBrokenWinCore* pBrkWinCore = (CBrokenWinCore*)::GetProp(hWnd,BRK_WIN_CLASS_PROP);
			_ASSERT(pBrkWinCore != NULL);
			pBrkWinCore->DeviceChangeEvent(wParam,lParam);
		}
		break;
	default:
		return ::DefWindowProc(hWnd, message, wParam, lParam);
	}

	return 0;
}

//      
void CBrokenWinCore::AddArea(CBrkWinArea* pBrkWinArea)
{
	//     
	pBrkWinArea->ProccesIntersect();
	//    
	m_BrkWinAreas.push_back(pBrkWinArea);
	//  
	pBrkWinArea->InitEvent();
}

//   
void CBrokenWinCore::ClearAreas()
{
	vector<CBrkWinArea*>::iterator BrkWinAreasMap_Iter = m_BrkWinAreas.begin();

	while (BrkWinAreasMap_Iter != m_BrkWinAreas.end())
	{
		if (*BrkWinAreasMap_Iter != NULL)
			delete *BrkWinAreasMap_Iter;
		BrkWinAreasMap_Iter = m_BrkWinAreas.erase(BrkWinAreasMap_Iter);
	}
}

//    
BOOL CBrokenWinCore::PreparationBGDC(HDC hDC, int BWcx, int BWcy)
{
	if (m_hWnd == NULL)
		return FALSE;

	//   
	if (BWcx == 0 || BWcy == 0)
		return FALSE;

	//    
	if (!m_hBGDC)
		m_hBGDC = CreateCompatibleDC(hDC);
	//    
	if (!m_hBGBitMap)
	{
		m_hBGBitMap = CreateCompatibleBitmap(hDC,BWcx,BWcy);
		m_hBGOldBitMap = (HBITMAP)SelectObject(m_hBGDC,m_hBGBitMap);
	}

	return TRUE;
}

//    
void CBrokenWinCore::DestroyBGDC()
{
	if (m_hBGDC != NULL)
	{
		if (m_hBGBitMap != NULL)
		{
			SelectObject(m_hBGDC,m_hBGOldBitMap);
			DeleteObject(m_hBGBitMap);
			m_hBGBitMap = NULL;
			m_hBGOldBitMap = NULL;
		}
		DeleteDC(m_hBGDC);
		m_hBGDC = NULL;
	}
}

//      
void CBrokenWinCore::DrawBGforArea(HDC hDC, POINT AreaPos, SIZE AreaSize)
{
	if (m_hBGDC == NULL)
		return;

	RECT AreaRect;
	AreaRect.left = 0;
	AreaRect.top = 0;
	AreaRect.right = AreaSize.cx;
	AreaRect.bottom = AreaSize.cy;

	//  
	HBRUSH hBkGndBrush = ::CreateSolidBrush(m_TransColor);
	::FillRect(hDC,&AreaRect,hBkGndBrush);
	::DeleteObject(hBkGndBrush);

	//  
	::BitBlt(hDC, 0, 0, AreaSize.cx, AreaSize.cy, m_hBGDC, AreaPos.x, AreaPos.y, SRCCOPY);
}

//      
int CBrokenWinCore::GetAreaIdx(WORD AreaID)
{
	int iAreaIdx = -1;

	for (UINT i = 0; i < m_BrkWinAreas.size(); i++)
	{
		if (m_BrkWinAreas[i]->GetAreaID() == AreaID)
			return i;
	}

	return iAreaIdx;
}

//    
CBrkWinArea* CBrokenWinCore::CreateArea(WORD AreaID, POINT AreaPos, SIZE AreaSize)
{
	//      
	PBWAREA_MAP_ENTRY pCreateAreasMap = GetCreateAreasMap();
	_ASSERT(pCreateAreasMap != NULL);

	while (pCreateAreasMap->pCreateAreaFunc != NULL)
	{
		if (pCreateAreasMap->AreaID == AreaID)
			return pCreateAreasMap->pCreateAreaFunc(AreaPos,AreaSize,this);

		//    
		pCreateAreasMap++;
	}

	return NULL;
}

//    
CBrkWinArea* CBrokenWinCore::GetArea(WORD AreaID)
{
	for (UINT i = 0; i < m_BrkWinAreas.size(); i++)
	{
		if (m_BrkWinAreas[i]->GetAreaID() == AreaID)
			return m_BrkWinAreas[i];
	}

	return NULL;
}

// 
BOOL CBrokenWinCore::Init(HINSTANCE hInstance, POINT BrkWinPos, COLORREF TransColor, double Scale, WORD BRKWIN_ID)
{
	// ---------------------------------------------------------
	//     
	// ---------------------------------------------------------
	CBrkWinResData BrkWinRes;
	if (!BrkWinRes.Load(hInstance,BRKWIN_ID))
		return FALSE;
	// 
	SIZE BrkWinSize;
	BrkWinRes.GetBrkWinSize(&BrkWinSize);
	BrkWinSize.cx = (LONG)ceil(Scale*BrkWinSize.cx - 0.5);
	BrkWinSize.cy = (LONG)ceil(Scale*BrkWinSize.cy - 0.5);
	//  
	m_TransColor = TransColor;
	//  
	UINT AreaCount = BrkWinRes.GetBrkWinAreaCount();

	// ---------------------------------------------------------
	//    
	// ---------------------------------------------------------
	if (m_hWnd != NULL)
	{//    
		//     
		::SetWindowPos(m_hWnd,NULL,BrkWinPos.x,BrkWinPos.y,BrkWinSize.cx,BrkWinSize.cy,SWP_NOZORDER | SWP_NOACTIVATE);
	}
	else
	{//    
		//   
		BrkWinRegisterClass(hInstance);
		m_hWnd = ::CreateWindowEx(WS_EX_LAYERED | WS_EX_TOOLWINDOW, BRK_WIN_CLASS_NAME, _T(""), WS_POPUP | WS_VISIBLE,
										  BrkWinPos.x, BrkWinPos.y, BrkWinSize.cx, BrkWinSize.cy, NULL, NULL, hInstance, this);
		if (m_hWnd == NULL)
			return FALSE;
	}
	//         
	SetLayeredWindowAttributes(m_hWnd,m_TransColor,0,LWA_COLORKEY);

	// ---------------------------------------------------------
	//   
	// ---------------------------------------------------------
	BROKEN_WIN_AREA_RES BrkWinArea;
	//     
	ClearAreas();
	//    
	CBrkWinArea* pArea = NULL;
	for (UINT i = 0; i < AreaCount; i++)
	{
		BrkWinRes.GetBrkWinArea(i,&BrkWinArea);
		//  
		POINT AreaPos;
		SIZE AreaSize;
		if (BrkWinArea.Style & BWA_NS_MASK)
		{//  
			if ((BrkWinArea.Style & BWA_NS_HA_MASK) == BWA_NS_LEFT)
			{//    
				AreaPos.x = (LONG)ceil(Scale*BrkWinArea.X0 - 0.5);
			}
			else if ((BrkWinArea.Style & BWA_NS_HA_MASK) == BWA_NS_HCENTER)
			{//    
				int CX_Half = BrkWinArea.CX / 2;
				AreaPos.x = (LONG)ceil(Scale*(BrkWinArea.X0 + CX_Half) - 0.5) - CX_Half;
			}
			else
			{//    
				AreaPos.x = (LONG)ceil(Scale*(BrkWinArea.X0 + BrkWinArea.CX) - 0.5) - BrkWinArea.CX;
			}

			if ((BrkWinArea.Style & BWA_NS_VA_MASK) == BWA_NS_TOP)
			{//    
				AreaPos.y = (LONG)ceil(Scale*BrkWinArea.Y0 - 0.5);
			}
			else if ((BrkWinArea.Style & BWA_NS_VA_MASK) == BWA_NS_VCENTER)
			{//    
				int CY_Half = BrkWinArea.CY / 2;
				AreaPos.y = (LONG)ceil(Scale*(BrkWinArea.Y0 + CY_Half) - 0.5) - CY_Half;
			}
			else
			{//    
				AreaPos.y = (LONG)ceil(Scale*(BrkWinArea.Y0 + BrkWinArea.CY) - 0.5) - BrkWinArea.CY;
			}

			AreaSize.cx = BrkWinArea.CX;
			AreaSize.cy = BrkWinArea.CY;
		}
		else
		{//  
			AreaPos.x = (LONG)ceil(Scale*BrkWinArea.X0 - 0.5);
			AreaPos.y = (LONG)ceil(Scale*BrkWinArea.Y0 - 0.5);
			AreaSize.cx = (LONG)ceil(Scale*(BrkWinArea.X0 + BrkWinArea.CX) - 0.5) - AreaPos.x;
			AreaSize.cy = (LONG)ceil(Scale*(BrkWinArea.Y0 + BrkWinArea.CY) - 0.5) - AreaPos.y;
		}
		pArea = CreateArea(BrkWinArea.AreaID,AreaPos,AreaSize);
		if (pArea != NULL)
		{//   
			//    
			AddArea(pArea);
		}
	}

	// ---------------------------------------------------------
	//   
	// ---------------------------------------------------------
	DestroyBGDC();
	::InvalidateRect(m_hWnd,NULL,TRUE);
	::UpdateWindow(m_hWnd);

	return TRUE;
}

//   
POINT CBrokenWinCore::GetAreaPos(WORD AreaID)
{
	POINT AreaPos;
	ZeroMemory(&AreaPos,sizeof(AreaPos));

	for (UINT i = 0; i < m_BrkWinAreas.size(); i++)
	{
		if (m_BrkWinAreas[i]->GetAreaID() == AreaID)
		{//   
			//  
			CBrkWinArea* pArea = m_BrkWinAreas[i];
			_ASSERT(pArea != NULL);
			//   
			AreaPos = pArea->m_Pos;
			break;
		}
	}

	return AreaPos;
}

//   
SIZE CBrokenWinCore::GetAreaSize(WORD AreaID)
{
	SIZE AreaSize;
	ZeroMemory(&AreaSize,sizeof(AreaSize));

	for (UINT i = 0; i < m_BrkWinAreas.size(); i++)
	{
		if (m_BrkWinAreas[i]->GetAreaID() == AreaID)
		{//   
			//  
			CBrkWinArea* pArea = m_BrkWinAreas[i];
			_ASSERT(pArea != NULL);
			//   
			AreaSize = pArea->m_Size;
			break;
		}
	}

	return AreaSize;
}

//   
void CBrokenWinCore::ReDrawBW()
{
	//   
	RECT ClRect;
	::GetClientRect(m_hWnd,&ClRect);
	//    
	if (!m_hWnd)
		return;
	HDC hBrkWinDC = ::GetDC(m_hWnd);
	if (!hBrkWinDC)
		return;

	//    
	DestroyBGDC();
	//     
	if (!PreparationBGDC(hBrkWinDC, ClRect.right, ClRect.bottom))
		return;
	//  
	DrawEvent(m_hBGDC);

	//   
	HDC hCompDC = ::CreateCompatibleDC(hBrkWinDC);
	HBITMAP hCompBitmap = ::CreateCompatibleBitmap(hBrkWinDC,ClRect.right,ClRect.bottom);
	HBITMAP hOldBitmap = (HBITMAP)::SelectObject(hCompDC, hCompBitmap);

	//     
	::BitBlt(hCompDC,0,0,ClRect.right,ClRect.bottom,m_hBGDC,0,0,SRCCOPY);
	//    
	vector<CBrkWinArea*>::iterator BrkWinAreasMap_Iter = m_BrkWinAreas.begin();
	while (BrkWinAreasMap_Iter != m_BrkWinAreas.end())
	{
		(*BrkWinAreasMap_Iter)->Draw(hCompDC,TRUE);
		BrkWinAreasMap_Iter++;
	}
	::BitBlt(hBrkWinDC,0,0,ClRect.right,ClRect.bottom,hCompDC,0,0,SRCCOPY);

	//  
	::SelectObject(hCompDC, hOldBitmap);
	::DeleteObject(hCompBitmap);
	::DeleteDC(hCompDC);
	::ReleaseDC(m_hWnd, hBrkWinDC);
}

//  
void CBrokenWinCore::ReDrawArea(WORD AreaID)
{
	for (UINT AreaIdx = 0; AreaIdx < m_BrkWinAreas.size(); AreaIdx++)
	{
		if (m_BrkWinAreas[AreaIdx]->GetAreaID() == AreaID)
		{//   
			//  
			CBrkWinArea* pArea = m_BrkWinAreas[AreaIdx];
			_ASSERT(pArea != NULL);
			//     
			POINT AreaPos = pArea->m_Pos;
			SIZE AreaSize = pArea->m_Size;
			RECT AreaRect;
			::SetRect(&AreaRect,AreaPos.x,AreaPos.y,AreaPos.x+AreaSize.cx,AreaPos.y+AreaSize.cy);
			if (::IsRectEmpty(&AreaRect))
				return;

			//    
			RECT ClRect;
			::GetClientRect(m_hWnd,&ClRect);
			//    
			if (!m_hWnd)
				return;
			HDC hBrkWinDC = ::GetDC(m_hWnd);
			if (!hBrkWinDC)
				return;

			//   
			HDC hCompDC = ::CreateCompatibleDC(hBrkWinDC);
			HBITMAP hCompBitmap = ::CreateCompatibleBitmap(hBrkWinDC,ClRect.right,ClRect.bottom);
			HBITMAP hOldBitmap = (HBITMAP)::SelectObject(hCompDC, hCompBitmap);

			//    
			for (UINT AboveAreaIdx = 0; AboveAreaIdx < m_BrkWinAreas.size(); AboveAreaIdx++)
			{
				//  
				CBrkWinArea* pAboveArea = m_BrkWinAreas[AboveAreaIdx];
				_ASSERT(pAboveArea != NULL);
				if (AboveAreaIdx == AreaIdx)
				{//   
					pAboveArea->Draw(hCompDC,TRUE);
				}
				if (AboveAreaIdx > AreaIdx)
				{//  ,   
					POINT AboveAreaPos = pAboveArea->m_Pos;
					SIZE AboveAreaSize = pAboveArea->m_Size;
					RECT AboveAreaRect;
					::SetRect(&AboveAreaRect,AboveAreaPos.x,AboveAreaPos.y,AboveAreaPos.x+AboveAreaSize.cx,AboveAreaPos.y+AboveAreaSize.cy);
					RECT InterRect;
					if (IntersectRect(&InterRect,&AreaRect,&AboveAreaRect))
					{//  
						pAboveArea->Draw(hCompDC,TRUE);
					}
				}
			}
			::BitBlt(hBrkWinDC,AreaPos.x,AreaPos.y,AreaSize.cx,AreaSize.cy,hCompDC,AreaPos.x,AreaPos.y,SRCCOPY);

			//  
			::SelectObject(hCompDC, hOldBitmap);
			::DeleteObject(hCompBitmap);
			::DeleteDC(hCompDC);
			::ReleaseDC(m_hWnd, hBrkWinDC);

			break;
		}
	}
}

//   
HWND CBrokenWinCore::GetWnd()
{
	return m_hWnd;
}

//   
void CBrokenWinCore::SetAreaCapture(WORD AreaID)
{
	//   ,  
	m_CaptureAreaIdx = GetAreaIdx(AreaID);
	_ASSERT(m_CaptureAreaIdx != -1);
	//  
	::SetCapture(m_hWnd);
}

//   ,  
WORD CBrokenWinCore::GetAreaCapture()
{
	if (m_CaptureAreaIdx != -1)
		return m_BrkWinAreas[m_CaptureAreaIdx]->GetAreaID();
	else
		return -1;
}

//  
void CBrokenWinCore::ReleaseAreaCapture(WORD AreaID)
{
	//  
	if (::GetCapture() == m_hWnd)
		::ReleaseCapture();
	//   ,  
	m_CaptureAreaIdx = -1;
}

//   
int CBrokenWinCore::Create(HWND hBrkWin, LPCREATESTRUCT lpCS)
{
	//    
	::SetProp(hBrkWin,BRK_WIN_CLASS_PROP,(void*)this);
	//   
	CreateEvent(hBrkWin,lpCS);

	return 0;
}

//  
void CBrokenWinCore::Destroy()
{
	//   
	::RemoveProp(m_hWnd,BRK_WIN_CLASS_PROP);
	//    
	for (UINT i = m_BrkWinAreas.size(); i > 0; i--)
	{
		m_BrkWinAreas[i-1]->DeInitEvent();
	}
	//   
	DestroyEvent();
	//     
	m_hWnd = NULL;
}

//   
void CBrokenWinCore::Draw(HDC hDC)
{
	//   
	RECT ClRect;
	::GetClientRect(m_hWnd,&ClRect);

	//  
	if (m_hBGDC == NULL)
	{//     
		//     
		if (!PreparationBGDC(hDC, ClRect.right, ClRect.bottom))
			return;
		//  
		DrawEvent(m_hBGDC);
	}

	//   
	HDC hCompDC = ::CreateCompatibleDC(hDC);
	HBITMAP hCompBitmap = ::CreateCompatibleBitmap(hDC,ClRect.right,ClRect.bottom);
	HBITMAP hOldBitmap = (HBITMAP)::SelectObject(hCompDC, hCompBitmap);

	//     
	::BitBlt(hCompDC,0,0,ClRect.right,ClRect.bottom,m_hBGDC,0,0,SRCCOPY);
	//    
	vector<CBrkWinArea*>::iterator BrkWinAreasMap_Iter = m_BrkWinAreas.begin();
	while (BrkWinAreasMap_Iter != m_BrkWinAreas.end())
	{
		(*BrkWinAreasMap_Iter)->Draw(hCompDC,FALSE);
		BrkWinAreasMap_Iter++;
	}
	::BitBlt(hDC,0,0,ClRect.right,ClRect.bottom,hCompDC,0,0,SRCCOPY);

	//  
	::SelectObject(hCompDC, hOldBitmap);
	::DeleteObject(hCompBitmap);
	::DeleteDC(hCompDC);
}

//     
void CBrokenWinCore::LButtonDown(BOOL fDoubleClick, UINT nFlags, POINT point)
{
	BOOL bStop = FALSE;

	for (UINT i = m_BrkWinAreas.size(); i > 0; i--)
	{
		if (m_BrkWinAreas[i-1]->PtInArea(point))
		{//    
			//     
			RECT AreaRect;
			m_BrkWinAreas[i-1]->GetAreaRect(&AreaRect);
			POINT AreaMousePt;
			AreaMousePt.x = point.x - AreaRect.left;
			AreaMousePt.y = point.y - AreaRect.top;
			//       
			bStop = m_BrkWinAreas[i-1]->LButtonDownEvent(fDoubleClick,nFlags,AreaMousePt);
		}
	}

	if (!bStop)
	{//     
		if (!fDoubleClick)
		{//    
			//   
			m_CapturePt.x = point.x;
			m_CapturePt.y = point.y;
			//  
			::SetCapture(m_hWnd);
		}
		//   
		LButtonDownEvent(fDoubleClick,nFlags,point);
	}
}

//  
void CBrokenWinCore::MouseMove(UINT nFlags, POINT point)
{
	BOOL bStop = FALSE;

	//       
	if (::GetCapture() == m_hWnd && m_CaptureAreaIdx == -1)
	{//     
		//      
		POINT MousePt;
		::GetCursorPos(&MousePt);
		//    
		POINT NewWinPos;
		NewWinPos.x = MousePt.x - m_CapturePt.x;
		NewWinPos.y = MousePt.y - m_CapturePt.y;
		//     
		::SetWindowPos(m_hWnd, NULL, NewWinPos.x, NewWinPos.y, 0, 0, SWP_NOZORDER | SWP_NOSIZE | SWP_NOACTIVATE);
	}
	else
	{
		//     
		if (::GetCapture() == m_hWnd && m_CaptureAreaIdx != -1)
		{//   
			//     
			RECT AreaRect;
			m_BrkWinAreas[m_CaptureAreaIdx]->GetAreaRect(&AreaRect);
			POINT AreaMousePt;
			AreaMousePt.x = point.x - AreaRect.left;
			AreaMousePt.y = point.y - AreaRect.top;
			//   
			bStop = m_BrkWinAreas[m_CaptureAreaIdx]->MouseMoveEvent(nFlags, AreaMousePt);
		}
		else
		{//   
			int iMoveNewAreaIdx = -1;

			for (UINT i = m_BrkWinAreas.size(); i > 0; i--)
			{
				if (m_BrkWinAreas[i - 1]->PtInArea(point))
				{//    
					iMoveNewAreaIdx = i - 1;
					//     
					RECT AreaRect;
					m_BrkWinAreas[iMoveNewAreaIdx]->GetAreaRect(&AreaRect);
					POINT AreaMousePt;
					AreaMousePt.x = point.x - AreaRect.left;
					AreaMousePt.y = point.y - AreaRect.top;
					//     
					if (m_iMovePrevAreaIdx != iMoveNewAreaIdx)
					{//     
						//       
						TRACKMOUSEEVENT TrkMsEv;
						TrkMsEv.cbSize = sizeof(TrkMsEv);
						TrkMsEv.dwFlags = TME_LEAVE;
						TrkMsEv.hwndTrack = m_hWnd;
						TrkMsEv.dwHoverTime = 0;
						::TrackMouseEvent(&TrkMsEv);
						//   "  "   
						if (m_iMovePrevAreaIdx != -1)
							m_BrkWinAreas[m_iMovePrevAreaIdx]->MouseOutArea();
						//   "  "   
						m_BrkWinAreas[iMoveNewAreaIdx]->MouseInArea(AreaMousePt);
						//   
						m_iMovePrevAreaIdx = iMoveNewAreaIdx;
					}
					//     
					bStop = m_BrkWinAreas[iMoveNewAreaIdx]->MouseMoveEvent(nFlags, AreaMousePt);
					break;
				}
			}

			//       ,   
			if (iMoveNewAreaIdx == -1)
			{//     ,   
				if (m_iMovePrevAreaIdx != iMoveNewAreaIdx)
				{
					//       
					TRACKMOUSEEVENT TrkMsEv;
					TrkMsEv.cbSize = sizeof(TrkMsEv);
					TrkMsEv.dwFlags = TME_LEAVE | TME_CANCEL;
					TrkMsEv.hwndTrack = m_hWnd;
					TrkMsEv.dwHoverTime = 0;
					::TrackMouseEvent(&TrkMsEv);
					//   "  "   
					if (m_iMovePrevAreaIdx != -1)
						m_BrkWinAreas[m_iMovePrevAreaIdx]->MouseOutArea();
					//   
					m_iMovePrevAreaIdx = -1;
				}
			}
		}
	}

	if (!bStop)
	{//     
		//   
		MouseMoveEvent(nFlags, point);
	}
}

//     
void CBrokenWinCore::MouseLeave()
{
	//   "  "   
	if (m_iMovePrevAreaIdx >= 0)
		m_BrkWinAreas[m_iMovePrevAreaIdx]->MouseOutArea();
	//   
	m_iMovePrevAreaIdx = -1;
}

//    
void CBrokenWinCore::LButtonUp(UINT nFlags, POINT point)
{
	BOOL bStop = FALSE;

	//       
	if (::GetCapture() == m_hWnd && m_CaptureAreaIdx == -1)
	{//     
		//   
		::ReleaseCapture();
	}
	else
	{
		//     
		if (::GetCapture() == m_hWnd && m_CaptureAreaIdx != -1)
		{//   
			//     
			RECT AreaRect;
			m_BrkWinAreas[m_CaptureAreaIdx]->GetAreaRect(&AreaRect);
			POINT AreaMousePt;
			AreaMousePt.x = point.x - AreaRect.left;
			AreaMousePt.y = point.y - AreaRect.top;
			//       
			bStop = m_BrkWinAreas[m_CaptureAreaIdx]->LButtonUpEvent(nFlags, AreaMousePt);
		}
		else
		{//   
			for (UINT i = m_BrkWinAreas.size(); i > 0; i--)
			{
				if (m_BrkWinAreas[i - 1]->PtInArea(point))
				{//    
					//     
					RECT AreaRect;
					m_BrkWinAreas[i - 1]->GetAreaRect(&AreaRect);
					POINT AreaMousePt;
					AreaMousePt.x = point.x - AreaRect.left;
					AreaMousePt.y = point.y - AreaRect.top;
					//       
					bStop = m_BrkWinAreas[i - 1]->LButtonUpEvent(nFlags, AreaMousePt);
				}
			}
		}
	}

	if (!bStop)
	{//     
		//   
		LButtonUpEvent(nFlags, point);
	}
}

//     
void CBrokenWinCore::RButtonDown(BOOL fDoubleClick, UINT nFlags, POINT point)
{
	BOOL bStop = FALSE;

	for (UINT i = m_BrkWinAreas.size(); i > 0; i--)
	{
		if (m_BrkWinAreas[i - 1]->PtInArea(point))
		{//    
			//     
			RECT AreaRect;
			m_BrkWinAreas[i - 1]->GetAreaRect(&AreaRect);
			POINT AreaMousePt;
			AreaMousePt.x = point.x - AreaRect.left;
			AreaMousePt.y = point.y - AreaRect.top;
			//       
			bStop = m_BrkWinAreas[i - 1]->RButtonDownEvent(fDoubleClick, nFlags, AreaMousePt);
		}
	}

	if (!bStop)
	{//     
		//   
		RButtonDownEvent(fDoubleClick, nFlags, point);
	}
}

//   
void CBrokenWinCore::ContextMenu(UINT xPos, UINT yPos)
{
	BOOL bStop = FALSE;

	//      
	POINT BrkWinPt;
	BrkWinPt.x = xPos;
	BrkWinPt.y = yPos;
	::ScreenToClient(m_hWnd, &BrkWinPt);

	for (UINT i = m_BrkWinAreas.size(); i > 0; i--)
	{
		if (m_BrkWinAreas[i - 1]->PtInArea(BrkWinPt))
		{//    
			//       
			bStop = m_BrkWinAreas[i - 1]->ContextMenuEvent(xPos, yPos);
		}
	}

	if (!bStop)
	{//     
		//   
		ContextMenuEvent(xPos, yPos);
	}
}
